@@ -146,6 +146,9 @@ local function process_server_args(args)
 end
 
 local function error_handler(err)
+   if not debug then
+      return err
+   end
    local mode = "Arch.: " .. (cfg and cfg.arch or "unknown")
    if package.config:sub(1, 1) == "\\" then
       if cfg and cfg.fs_use_modules then
@@ -460,6 +463,7 @@ Enabling completion for Fish:
       :argname("<prefix>")
    parser:option("--lua-version", "Which Lua version to use.")
       :argname("<ver>")
+      :convert(function(s) return (s:match("^%d+%.%d+$")) end)
    parser:option("--tree", "Which tree to operate on.")
       :hidden_name("--to")
    parser:flag("--local", "Use the tree in the user's home directory.\n"..
