@@ -10,18 +10,17 @@
 -- files. Run `luarocks` with no arguments to see the locations of
 -- these files in your platform.
 
-local next, table, pairs, require, os, pcall, ipairs, package, tonumber, type, assert =
-      next, table, pairs, require, os, pcall, ipairs, package, tonumber, type, assert
+local table, pairs, require, os, pcall, ipairs, package, type, assert =
+      table, pairs, require, os, pcall, ipairs, package, type, assert
 
 local util = require("luarocks.core.util")
 local persist = require("luarocks.core.persist")
 local sysdetect = require("luarocks.core.sysdetect")
+local vers = require("luarocks.core.vers")
 
 --------------------------------------------------------------------------------
 
-local program_version = "3.5.0"
-local program_series = "3.5"
-local major_version = (program_version:match("([^.]%.[^.])")) or program_series
+local program_version = "3.9.0"
 
 local is_windows = package.config:sub(1,1) == "\\"
 
@@ -36,6 +35,7 @@ local platform_order = {
    "netbsd",
    "openbsd",
    "freebsd",
+   "dragonfly",
    "linux",
    "macosx",
    "cygwin",
@@ -44,11 +44,15 @@ local platform_order = {
    -- Windows
    "windows",
    "win32",
+   "mingw",
    "mingw32",
    "msys2_mingw_w64",
 }
 
 local function detect_sysconfdir()
+   if not debug then
+      return
+   end
    local src = debug.getinfo(1, "S").source:gsub("\\", "/"):gsub("/+", "/")
    if src:sub(1, 1) == "@" then
       src = src:sub(2)
@@ -69,37 +73,20 @@ local function detect_sysconfdir()
    return basedir
 end
 
-local function set_confdirs(cfg, platforms, hardcoded_sysconfdir)
-   local sysconfdir = os.getenv("LUAROCKS_SYSCONFDIR") or hardcoded_sysconfdir
-   local windows_style = platforms.windows
-   if platforms.msys2_mingw_w64 then
-      windows_style = false
-   end
-   if windows_style then
-      cfg.home = os.getenv("APPDATA") or "c:"
-      cfg.home_tree = cfg.home.."/luarocks"
-      cfg.homeconfdir = cfg.home_tree
-      cfg.sysconfdir = sysconfdir or ((os.getenv("PROGRAMFILES") or "c:") .. "/luarocks")
-   else
-      if not sysconfdir then
-         sysconfdir = detect_sysconfdir()
-      end
-      cfg.home = os.getenv("HOME") or ""
-      cfg.home_tree = (os.getenv("USER") ~= "root") and cfg.home.."/.luarocks"
-      cfg.homeconfdir = cfg.home.."/.luarocks"
-      cfg.sysconfdir = sysconfdir or "/etc/luarocks"
-   end
-end
-
 local load_config_file
 do
    -- Create global environment for the config files;
    local function env_for_config_file(cfg, platforms)
+      local platforms_copy = {}
+      for k,v in pairs(platforms) do
+         platforms_copy[k] = v
+      end
+
       local e
       e = {
          home = cfg.home,
          lua_version = cfg.lua_version,
-         platforms = util.make_shallow_copy(platforms),
+         platforms = platforms_copy,
          processor = cfg.target_cpu,   -- remains for compat reasons
          target_cpu = cfg.target_cpu,  -- replaces `processor`
          os_getenv = os.getenv,
@@ -163,6 +150,7 @@ end
 local platform_sets = {
    freebsd = { unix = true, bsd = true, freebsd = true },
    openbsd = { unix = true, bsd = true, openbsd = true },
+   dragonfly = { unix = true, bsd = true, dragonfly = true },
    solaris = { unix = true, solaris = true },
    windows = { windows = true, win32 = true },
    cygwin = { unix = true, cygwin = true },
@@ -213,7 +201,6 @@ local function make_defaults(lua_version, target_cpu, platforms, home)
            "https://luarocks.org",
            "https://raw.githubusercontent.com/rocks-moonscript-org/moonrocks-mirror/master/",
            "https://luafr.org/luarocks/",
-           "http://luarocks.logiceditor.com/rocks",
          }
       },
       disabled_servers = {},
@@ -299,6 +286,7 @@ local function make_defaults(lua_version, target_cpu, platforms, home)
       defaults.external_deps_dirs = { "c:/external/", "c:/windows/system32" }
 
       defaults.makefile = "Makefile.win"
+      defaults.variables.PWD = "echo %cd%"
       defaults.variables.MAKE = "nmake"
       defaults.variables.CC = "cl"
       defaults.variables.RC = "rc"
@@ -306,7 +294,8 @@ local function make_defaults(lua_version, target_cpu, platforms, home)
       defaults.variables.MT = "mt"
       defaults.variables.AR = "lib"
       defaults.variables.LUALIB = "lua"..lua_version..".lib"
-      defaults.variables.CFLAGS = "/nologo /MD /O2"
+      defaults.variables.CFLAGS = os.getenv("CFLAGS") or "/nologo /MD /O2"
+      defaults.variables.LDFLAGS = os.getenv("LDFLAGS")
       defaults.variables.LIBFLAG = "/nologo /dll"
 
       defaults.external_deps_patterns = {
@@ -347,7 +336,8 @@ local function make_defaults(lua_version, target_cpu, platforms, home)
       defaults.variables.LD = "mingw32-gcc"
       defaults.variables.AR = "ar"
       defaults.variables.RANLIB = "ranlib"
-      defaults.variables.CFLAGS = "-O2"
+      defaults.variables.CFLAGS = os.getenv("CFLAGS") or "-O2"
+      defaults.variables.LDFLAGS = os.getenv("LDFLAGS")
       defaults.variables.LIBFLAG = "-shared"
       defaults.makefile = "Makefile"
       defaults.external_deps_patterns = {
@@ -371,7 +361,16 @@ local function make_defaults(lua_version, target_cpu, platforms, home)
       defaults.external_lib_extension = "so"
       defaults.obj_extension = "o"
       defaults.external_deps_dirs = { "/usr/local", "/usr", "/" }
-      defaults.variables.CFLAGS = "-O2"
+
+      defaults.variables.CFLAGS = os.getenv("CFLAGS") or "-O2"
+      -- we pass -fPIC via CFLAGS because of old Makefile-based Lua projects
+      -- which didn't have -fPIC in their Makefiles but which honor CFLAGS
+      if not defaults.variables.CFLAGS:match("-fPIC") then
+         defaults.variables.CFLAGS = defaults.variables.CFLAGS.." -fPIC"
+      end
+
+      defaults.variables.LDFLAGS = os.getenv("LDFLAGS")
+
       defaults.cmake_generator = "Unix Makefiles"
       defaults.variables.CC = "gcc"
       defaults.variables.LD = "gcc"
@@ -391,10 +390,8 @@ local function make_defaults(lua_version, target_cpu, platforms, home)
       }
       defaults.export_path_separator = ":"
       defaults.wrapper_suffix = ""
-      defaults.local_cache = home.."/.cache/luarocks"
-      if not defaults.variables.CFLAGS:match("-fPIC") then
-         defaults.variables.CFLAGS = defaults.variables.CFLAGS.." -fPIC"
-      end
+      local xdg_cache_home = os.getenv("XDG_CACHE_HOME") or home.."/.cache"
+      defaults.local_cache = xdg_cache_home.."/luarocks"
       defaults.web_browser = "xdg-open"
    end
 
@@ -442,13 +439,21 @@ local function make_defaults(lua_version, target_cpu, platforms, home)
          defaults.variables.AR = "ar"
          defaults.variables.RANLIB = "ranlib"
          defaults.variables.LUALIB = "liblua"..lua_version..".dll.a"
-         defaults.variables.CFLAGS = "-O2 -fPIC"
+
+         defaults.variables.CFLAGS = os.getenv("CFLAGS") or "-O2 -fPIC"
+         if not defaults.variables.CFLAGS:match("-fPIC") then
+            defaults.variables.CFLAGS = defaults.variables.CFLAGS.." -fPIC"
+         end
+
          defaults.variables.LIBFLAG = "-shared"
       end
    end
 
    if platforms.bsd then
       defaults.variables.MAKE = "gmake"
+      defaults.gcc_rpath = false
+      defaults.variables.CC = "cc"
+      defaults.variables.LD = defaults.variables.CC
    end
 
    if platforms.macosx then
@@ -457,17 +462,37 @@ local function make_defaults(lua_version, target_cpu, platforms, home)
       defaults.arch = "macosx-"..target_cpu
       defaults.variables.LIBFLAG = "-bundle -undefined dynamic_lookup -all_load"
       local version = util.popen_read("sw_vers -productVersion")
-      version = tonumber(version and version:match("^[^.]+%.([^.]+)")) or 3
-      if version >= 10 then
-         version = 8
-      elseif version >= 5 then
-         version = 5
+      if not (version:match("^%d+%.%d+%.%d+$") or version:match("^%d+%.%d+$")) then
+         version = "10.3"
+      end
+      version = vers.parse_version(version)
+      if version >= vers.parse_version("11.0") then
+         version = vers.parse_version("11.0")
+      elseif version >= vers.parse_version("10.10") then
+         version = vers.parse_version("10.8")
+      elseif version >= vers.parse_version("10.5") then
+         version = vers.parse_version("10.5")
       else
          defaults.gcc_rpath = false
       end
-      defaults.variables.CC = "env MACOSX_DEPLOYMENT_TARGET=10."..version.." gcc"
-      defaults.variables.LD = "env MACOSX_DEPLOYMENT_TARGET=10."..version.." gcc"
+      defaults.variables.CC = "env MACOSX_DEPLOYMENT_TARGET="..tostring(version).." gcc"
+      defaults.variables.LD = "env MACOSX_DEPLOYMENT_TARGET="..tostring(version).." gcc"
       defaults.web_browser = "open"
+
+      -- XCode SDK
+      local sdk_path = util.popen_read("xcrun --show-sdk-path 2>/dev/null")
+      if sdk_path then
+         table.insert(defaults.external_deps_dirs, sdk_path .. "/usr")
+         table.insert(defaults.external_deps_patterns.lib, 1, "lib?.tbd")
+         table.insert(defaults.runtime_external_deps_patterns.lib, 1, "lib?.tbd")
+      end
+
+      -- Homebrew
+      table.insert(defaults.external_deps_dirs, "/usr/local/opt")
+      defaults.external_deps_subdirs.lib = { "", "lib", }
+      defaults.runtime_external_deps_subdirs.lib = { "", "lib", }
+      table.insert(defaults.external_deps_patterns.lib, 1, "/?/lib/lib?.dylib")
+      table.insert(defaults.runtime_external_deps_patterns.lib, 1, "/?/lib/lib?.dylib")
    end
 
    if platforms.linux then
@@ -485,22 +510,14 @@ local function make_defaults(lua_version, target_cpu, platforms, home)
 
    if platforms.freebsd then
       defaults.arch = "freebsd-"..target_cpu
-      defaults.gcc_rpath = false
-      defaults.variables.CC = "cc"
-      defaults.variables.LD = "cc"
-   end
-
-   if platforms.openbsd then
+   elseif platforms.dragonfly then
+      defaults.arch = "dragonfly-"..target_cpu
+   elseif platforms.openbsd then
       defaults.arch = "openbsd-"..target_cpu
-   end
-
-   if platforms.netbsd then
+   elseif platforms.netbsd then
       defaults.arch = "netbsd-"..target_cpu
-   end
-
-   if platforms.solaris then
+   elseif platforms.solaris then
       defaults.arch = "solaris-"..target_cpu
-      --defaults.platforms = {"unix", "solaris"}
       defaults.variables.MAKE = "gmake"
    end
 
@@ -552,7 +569,7 @@ local cfg = {}
 
 --- Initializes the LuaRocks configuration for variables, paths
 -- and OS detection.
--- @param detected table containing information detected about the 
+-- @param detected table containing information detected about the
 -- environment. All fields below are optional:
 -- * lua_version (in x.y format, e.g. "5.3")
 -- * lua_bindir (e.g. "/usr/local/bin")
@@ -565,11 +582,15 @@ local cfg = {}
 function cfg.init(detected, warning)
    detected = detected or {}
 
+   local exit_ok = true
+   local exit_err = nil
+   local exit_what = nil
+
    local hc_ok, hardcoded = pcall(require, "luarocks.core.hardcoded")
    if not hc_ok then
       hardcoded = {}
    end
-   
+
    local init = cfg.init
 
    ----------------------------------------
@@ -581,15 +602,13 @@ function cfg.init(detected, warning)
    end
 
    cfg.program_version = program_version
-   cfg.program_series = program_series
-   cfg.major_version = major_version
 
    if hardcoded.IS_BINARY then
       cfg.is_binary = true
    end
 
    -- Use detected values as defaults, overridable via config files or CLI args
-   
+
    local first_arg = get_first_arg()
 
    cfg.lua_version = detected.lua_version or hardcoded.LUA_VERSION or _VERSION:sub(5)
@@ -653,20 +672,27 @@ function cfg.init(detected, warning)
    local sys_config_file
    local home_config_file
    local project_config_file
+
+   local config_file_name = "config-"..cfg.lua_version..".lua"
+
    do
-      set_confdirs(cfg, platforms, hardcoded.SYSCONFDIR)
-      local name = "config-"..cfg.lua_version..".lua"
-      sys_config_file = (cfg.sysconfdir .. "/" .. name):gsub("\\", "/")
-      home_config_file = (cfg.homeconfdir .. "/" .. name):gsub("\\", "/")
-      if cfg.project_dir then
-         project_config_file = cfg.project_dir .. "/.luarocks/" .. name
+      local sysconfdir = os.getenv("LUAROCKS_SYSCONFDIR") or hardcoded.SYSCONFDIR
+      if platforms.windows and not platforms.msys2_mingw_w64 then
+         cfg.home = os.getenv("APPDATA") or "c:"
+         cfg.home_tree = cfg.home.."/luarocks"
+         cfg.sysconfdir = sysconfdir or ((os.getenv("PROGRAMFILES") or "c:") .. "/luarocks")
+      else
+         cfg.home = os.getenv("HOME") or ""
+         cfg.home_tree = cfg.home.."/.luarocks"
+         cfg.sysconfdir = sysconfdir or detect_sysconfdir() or "/etc/luarocks"
       end
    end
 
    -- Load system configuration file
+   sys_config_file = (cfg.sysconfdir .. "/" .. config_file_name):gsub("\\", "/")
    local sys_config_ok, err = load_config_file(cfg, platforms, sys_config_file)
    if err then
-      return nil, err, "config"
+      exit_ok, exit_err, exit_what = nil, err, "config"
    end
 
    -- Load user configuration file (if allowed)
@@ -683,7 +709,7 @@ function cfg.init(detected, warning)
       if env_value then
          local env_ok, err = load_config_file(cfg, platforms, env_value)
          if err then
-            return nil, err, "config"
+            exit_ok, exit_err, exit_what = nil, err, "config"
          elseif warning and not env_ok then
             warning("Warning: could not load configuration file `"..env_value.."` given in environment variable "..env_var.."\n")
          end
@@ -693,19 +719,33 @@ function cfg.init(detected, warning)
          end
       end
 
-      -- try the alternative defaults if there was no environment specified file or it didn't work
-      if not home_config_ok then
+      -- try XDG config home
+      if platforms.unix and not home_config_ok then
+         local xdg_config_home = os.getenv("XDG_CONFIG_HOME") or cfg.home .. "/.config"
+         cfg.homeconfdir = xdg_config_home .. "/luarocks"
+         home_config_file = (cfg.homeconfdir .. "/" .. config_file_name):gsub("\\", "/")
          home_config_ok, err = load_config_file(cfg, platforms, home_config_file)
          if err then
-            return nil, err, "config"
+            exit_ok, exit_err, exit_what = nil, err, "config"
+         end
+      end
+
+      -- try the alternative defaults if there was no environment specified file or it didn't work
+      if not home_config_ok then
+         cfg.homeconfdir = cfg.home_tree
+         home_config_file = (cfg.homeconfdir .. "/" .. config_file_name):gsub("\\", "/")
+         home_config_ok, err = load_config_file(cfg, platforms, home_config_file)
+         if err then
+            exit_ok, exit_err, exit_what = nil, err, "config"
          end
       end
 
       -- finally, use the project-specific config file if any
       if cfg.project_dir then
+         project_config_file = cfg.project_dir .. "/.luarocks/" .. config_file_name
          project_config_ok, err = load_config_file(cfg, platforms, project_config_file)
          if err then
-            return nil, err, "config"
+            exit_ok, exit_err, exit_what = nil, err, "config"
          end
       end
    end
@@ -740,7 +780,7 @@ function cfg.init(detected, warning)
    local defaults = make_defaults(cfg.lua_version, processor, platforms, cfg.home)
 
    if platforms.windows and hardcoded.WIN_TOOLS then
-      local tools = { "SEVENZ", "CP", "FIND", "LS", "MD5SUM", "PWD", "RMDIR", "WGET", "MKDIR" }
+      local tools = { "SEVENZ", "CP", "FIND", "LS", "MD5SUM", "WGET", }
       for _, tool in ipairs(tools) do
          defaults.variables[tool] = '"' .. hardcoded.WIN_TOOLS .. "/" .. defaults.variables[tool] .. '.exe"'
       end
@@ -772,7 +812,7 @@ function cfg.init(detected, warning)
                     and home_config_file
                     or sys_config_file),
    }
-   
+
    cfg.cache = {}
 
    ----------------------------------------
@@ -858,7 +898,7 @@ function cfg.init(detected, warning)
       return table.concat(platform_keys, ", ")
    end
 
-   return true
+   return exit_ok, exit_err, exit_what
 end
 
 return cfg
