@@ -223,7 +223,7 @@ function deps.fulfill_dependency(dep, deps_mode, rocks_provided, verify, depskey
    local search = require("luarocks.search")
    local install = require("luarocks.cmd.install")
 
-   local url, search_err = search.find_suitable_rock(dep, true)
+   local url, search_err = search.find_suitable_rock(dep)
    if not url then
       return nil, "Could not satisfy dependency "..tostring(dep)..": "..search_err
    end
@@ -373,7 +373,7 @@ end
 -- @param files The array of constructed names
 local function add_all_patterns(file, patterns, files)
    for _, pattern in ipairs(patterns) do
-      table.insert(files, (pattern:gsub("?", file)))
+      table.insert(files, {#files + 1, (pattern:gsub("?", file))})
    end
 end
 
@@ -425,7 +425,7 @@ local function add_patterns_for_file(files, file, patterns)
             add_all_patterns(matched, patterns, files)
          end
       end
-      table.insert(files, file)
+      table.insert(files, {#files + 1, file})
    end
 end
 
@@ -460,16 +460,17 @@ local function check_external_dependency_at(prefix, name, ext_files, vars, dirs,
 
          local found = false
          table.sort(files, function(a, b)
-            if (not a:match("%*")) and b:match("%*") then
+            if (not a[2]:match("%*")) and b[2]:match("%*") then
                return true
-            elseif a:match("%*") and (not b:match("%*")) then
+            elseif a[2]:match("%*") and (not b[2]:match("%*")) then
                return false
             else
-               return a < b
+               return a[1] < b[1]
             end
          end)
-         for _, f in ipairs(files) do
+         for _, fa in ipairs(files) do
 
+            local f = fa[2]
             -- small convenience hack
             if f:match("%.so$") or f:match("%.dylib$") or f:match("%.dll$") then
                f = f:gsub("%.[^.]+$", "."..cfg.external_lib_extension)
@@ -664,11 +665,10 @@ local function lua_h_exists(d, luaver)
       if data:match("LUA_VERSION_NUM%s*" .. tostring(luanum)) then
          return d
       end
-
-      return nil, "Lua header mismatches configured version. You may need to install them or configure LUA_INCDIR.", "dependency"
+      return nil, "Lua header found at " .. d .. " does not match Lua version " .. luaver .. ". You may want to override this by configuring LUA_INCDIR.", "dependency", 2
    end
 
-   return nil, "Failed finding Lua header files. You may need to install them or configure LUA_INCDIR.", "dependency"
+   return nil, "Failed finding Lua header files. You may need to install them or configure LUA_INCDIR.", "dependency", 1
 end
 
 local function find_lua_incdir(prefix, luaver, luajitver)
@@ -683,14 +683,21 @@ local function find_lua_incdir(prefix, luaver, luajitver)
       prefix,
       luajitver and prefix .. "/include/luajit-" .. luajitver:match("^(%d+%.%d+)"),
    }
+   local errprio = 0
+   local mainerr
    for _, d in ipairs(incdirs) do
-      if lua_h_exists(d, luaver) then
+      local ok, err, _, prio = lua_h_exists(d, luaver)
+      if ok then
          return d
       end
+      if prio > errprio then
+         mainerr = err
+         errprio = prio
+      end
    end
 
    -- not found, will fallback to a default
-   return nil
+   return nil, mainerr
 end
 
 function deps.check_lua_incdir(vars)
@@ -701,18 +708,25 @@ function deps.check_lua_incdir(vars)
    end
 
    if vars.LUA_DIR then
-      vars.LUA_INCDIR = find_lua_incdir(vars.LUA_DIR, cfg.lua_version, ljv)
-      if vars.LUA_INCDIR then
+      local d, err = find_lua_incdir(vars.LUA_DIR, cfg.lua_version, ljv)
+      if d then
+         vars.LUA_INCDIR = d
          return true
       end
+      return nil, err
    end
 
    return nil, "Failed finding Lua header files. You may need to install them or configure LUA_INCDIR.", "dependency"
 end
 
 function deps.check_lua_libdir(vars)
+   local fs = require("luarocks.fs")
    local ljv = util.get_luajit_version()
 
+   if vars.LUA_LIBDIR and vars.LUALIB and fs.exists(dir.path(vars.LUA_LIBDIR, vars.LUALIB)) then
+      return true
+   end
+
    local shortv = cfg.lua_version:gsub("%.", "")
    local libnames = {
       "lua" .. cfg.lua_version,
@@ -728,11 +742,32 @@ function deps.check_lua_libdir(vars)
    local save_LUA_INCDIR = vars.LUA_INCDIR
    local ok = check_external_dependency("LUA", { library = libnames }, vars, "build", cache)
    vars.LUA_INCDIR = save_LUA_INCDIR
+   local err
+   if ok then
+      local filename = dir.path(vars.LUA_LIBDIR, vars.LUA_LIBDIR_FILE)
+      local fd = io.open(filename, "r")
+      if fd then
+         if not vars.LUA_LIBDIR_FILE:match((cfg.lua_version:gsub("%.", "%%.?"))) then
+            -- if filename isn't versioned, check file contents
+            local txt = fd:read("*a")
+            ok = txt:match("Lua " .. cfg.lua_version, 1, true)
+                 or txt:match("lua" .. (cfg.lua_version:gsub("%.", "")), 1, true)
+            if not ok then
+               err = "Lua library at " .. filename .. " does not match Lua version " .. cfg.lua_version .. ". You may want to override this by configuring LUA_INCDIR."
+            end
+         end
+
+         fd:close()
+      end
+   end
+
    if ok then
       vars.LUALIB = vars.LUA_LIBDIR_FILE
       return true
+   else
+      err = err or "Failed finding Lua library. You may need to configure LUA_LIBDIR."
+      return nil, err, "dependency"
    end
-   return nil, "Failed finding Lua library. You may need to configure LUA_LIBDIR.", "dependency"
 end
 
 function deps.get_deps_mode(args)
